@@ -9,14 +9,14 @@
 extern "C" {
 #endif
 
-void kinesis_init(const char *region, const char *access_key_id, const char *secret_key, const long timeout);
+void backends_kinesis_init(const char *region, const char *access_key_id, const char *secret_key, const long timeout);
 
-void kinesis_shutdown();
+void backends_kinesis_shutdown();
 
-int kinesis_put_record(const char *stream_name, const char *partition_key,
+int backends_kinesis_put_record(const char *stream_name, const char *partition_key,
                        const char *data, size_t data_len);
 
-int kinesis_get_result(char *error_message, size_t *sent_bytes, size_t *lost_bytes);
+int backends_kinesis_get_result(char *error_message, size_t *sent_bytes, size_t *lost_bytes);
 
 #ifdef __cplusplus
 }
