@@ -282,6 +282,8 @@
 #define NETDATA_CHART_PRIO_TC_QOS_TOCKENS             7030
 #define NETDATA_CHART_PRIO_TC_QOS_CTOCKENS            7040
 
+// Infiniband
+#define NETDATA_CHART_PRIO_INFINIBAND                 7100
 
 // Netfilter
 
@@ -310,11 +312,17 @@
 #define NETDATA_CHART_PRIO_MDSTAT_SPEED               9006
 
 // Linux Power Supply
+
 #define NETDATA_CHART_PRIO_POWER_SUPPLY_CAPACITY      9500 // 4 charts per power supply
 #define NETDATA_CHART_PRIO_POWER_SUPPLY_CHARGE        9501
 #define NETDATA_CHART_PRIO_POWER_SUPPLY_ENERGY        9502
 #define NETDATA_CHART_PRIO_POWER_SUPPLY_VOLTAGE       9503
 
+
+// Wireless
+
+#define NETDATA_CHART_PRIO_WIRELESS_IFACE	     7110
+
 // CGROUPS
 
 #define NETDATA_CHART_PRIO_CGROUPS_SYSTEMD           19000 // many charts
