@@ -10,7 +10,10 @@ char *netdata_configured_primary_plugins_dir = NULL;
 char *netdata_configured_web_dir             = WEB_DIR;
 char *netdata_configured_cache_dir           = CACHE_DIR;
 char *netdata_configured_varlib_dir          = VARLIB_DIR;
-char *netdata_configured_home_dir            = CACHE_DIR;
+char *netdata_configured_lock_dir            = NULL;
+char *netdata_configured_home_dir            = VARLIB_DIR;
 char *netdata_configured_host_prefix         = NULL;
 char *netdata_configured_timezone            = NULL;
+int netdata_ready;
+int netdata_cloud_setting;
 
