@@ -45,6 +45,13 @@ struct rrdcalc {
     char *chart;                    // the chart id this should be linked to
     uint32_t hash_chart;
 
+
+    char *plugin_match;             //the plugin name that should be linked to
+    SIMPLE_PATTERN *plugin_pattern;
+
+    char *module_match;             //the module name that should be linked to
+    SIMPLE_PATTERN *module_pattern;
+
     char *source;                   // the source of this alarm
     char *units;                    // the units of the alarm
     char *info;                     // a short description of the alarm
@@ -90,6 +97,11 @@ struct rrdcalc {
     uint32_t warn_repeat_every;     // interval between repeating warning notifications
     uint32_t crit_repeat_every; // interval between repeating critical notifications
 
+    // ------------------------------------------------------------------------
+    // Labels settings
+    char *labels;                   // the label read from an alarm file
+    SIMPLE_PATTERN *splabels;       // the simple pattern of labels
+
     // ------------------------------------------------------------------------
     // runtime information
 
@@ -157,6 +169,9 @@ extern void rrdcalc_add_to_host(RRDHOST *host, RRDCALC *rc);
 extern void dimension_remove_pipe_comma(char *str);
 extern char *alarm_name_with_dim(char *name, size_t namelen, const char *dim, size_t dimlen);
 
+extern void rrdcalc_labels_unlink();
+extern void rrdcalc_labels_unlink_alarm_from_host(RRDHOST *host);
+
 static inline int rrdcalc_isrepeating(RRDCALC *rc) {
     if (unlikely(rc->warn_repeat_every > 0 || rc->crit_repeat_every > 0)) {
         return 1;
