@@ -1,5 +1,7 @@
 #include "health.h"
 
+SILENCERS *silencers;
+
 /**
  * Create Silencer
  *
@@ -136,8 +138,8 @@ int health_silencers_json_read_callback(JSON_ENTRY *e)
                 else if (!strcmp(e->data.string,"DISABLE")) silencers->stype = STYPE_DISABLE_ALARMS;
             } else {
                 debug(D_HEALTH, "JSON: Adding %s=%s", e->name, e->data.string);
-                SILENCER *test = health_silencers_addparam(e->callback_data, e->name, e->data.string);
-                (void)test;
+                if (e->callback_data)
+                    (void)health_silencers_addparam(e->callback_data, e->name, e->data.string);
             }
             break;
 
