@@ -168,6 +168,7 @@ char *strdupz(const char *s) {
     return t;
 }
 
+// If ptr is NULL, no operation is performed.
 void freez(void *ptr) {
     free(ptr);
 }
@@ -1405,7 +1406,7 @@ void recursive_config_double_dir_load(const char *user_path, const char *stock_p
     if (!dir) {
         error("CONFIG cannot open stock config directory '%s'.", sdir);
     }
-    else {
+    else if (strcmp(udir, sdir)) {
         struct dirent *de = NULL;
         while((de = readdir(dir))) {
             if(de->d_type == DT_DIR || de->d_type == DT_LNK) {
@@ -1452,3 +1453,41 @@ void recursive_config_double_dir_load(const char *user_path, const char *stock_p
     freez(udir);
     freez(sdir);
 }
+
+// Returns the number of bytes read from the file if file_size is not NULL.
+// The actual buffer has an extra byte set to zero (not included in the count).
+char *read_by_filename(char *filename, long *file_size)
+{
+    FILE *f = fopen(filename, "r");
+    if (!f)
+        return NULL;
+    if (fseek(f, 0, SEEK_END) < 0) {
+        fclose(f);
+        return NULL;
+    }
+    long size = ftell(f);
+    if (size <= 0 || fseek(f, 0, SEEK_END) < 0) {
+        fclose(f);
+        return NULL;
+    }
+    char *contents = callocz(size + 1, 1);
+    if (!contents) {
+        fclose(f);
+        return NULL;
+    }
+    if (fseek(f, 0, SEEK_SET) < 0) {
+        fclose(f);
+        freez(contents);
+        return NULL;
+    }
+    size_t res = fread(contents, 1, size, f);
+    if ( res != (size_t)size) {
+        freez(contents);
+        fclose(f);
+        return NULL;
+    }
+    fclose(f);
+    if (file_size)
+        *file_size = size;
+    return contents;
+}
