@@ -32,5 +32,9 @@ extern void simple_pattern_free(SIMPLE_PATTERN *list);
 
 extern void simple_pattern_dump(uint64_t debug_type, SIMPLE_PATTERN *p) ;
 extern int simple_pattern_is_potential_name(SIMPLE_PATTERN *p) ;
+extern char *simple_pattern_iterate(SIMPLE_PATTERN **p);
+
+//Auxiliary function to create a pattern
+char *simple_pattern_trim_around_equal(char *src);
 
 #endif //NETDATA_SIMPLE_PATTERN_H
