@@ -8,6 +8,18 @@
 #include <cmocka.h>
 #include <stdbool.h>
 
+RRDHOST *sql_create_host_by_uuid(char *hostname)
+{
+    (void) hostname;
+    return NULL;
+}
+
+RRDHOST *__wrap_sql_create_host_by_uuid(char *hostname)
+{
+    (void) hostname;
+    return NULL;
+}
+
 void repr(char *result, int result_size, char const *buf, int size)
 {
     int n;
@@ -182,7 +194,8 @@ WEB_SERVER_MODE web_server_mode = WEB_SERVER_MODE_STATIC_THREADED;
 char *netdata_configured_web_dir = "UNKNOWN FIXME";
 RRDHOST *localhost = NULL;
 
-struct config netdata_config = { .sections = NULL,
+struct config netdata_config = { .first_section = NULL,
+                                 .last_section = NULL,
                                  .mutex = NETDATA_MUTEX_INITIALIZER,
                                  .index = { .avl_tree = { .root = NULL, .compar = appconfig_section_compare },
                                             .rwlock = AVL_LOCK_INITIALIZER } };
